@@ -16,6 +16,7 @@ const (
 )
 
 var ErrPrebuiltLoaderSetNotSupported = fmt.Errorf("dyld_shared_cache has no launch prebuilt loader set info")
+var ErrProgramTrieNotSupported = fmt.Errorf("dyld_shared_cache has no program trie info")
 
 type LoaderRef uint16
 
@@ -59,8 +60,10 @@ type Loader struct {
 	// hasReadOnlyObjC    :  1,  // Has __DATA_CONST,__objc_selrefs section
 	// pre2022Binary      :  1,
 	// isPremapped        :  1,  // mapped by exclave core
+	// isVersion2         :  1,  // FIXME: when dyld src is out for iOS 18.0/macOS 15.0
 	// padding            :  6;
 	Ref LoaderRef
+	// Unk [12]uint16
 }
 
 func (l Loader) IsPrebuilt() bool {
@@ -93,6 +96,9 @@ func (l Loader) Pre2022Binary() bool {
 func (l Loader) IsPremapped() bool {
 	return types.ExtractBits(uint64(l.Info), 9, 1) != 0
 }
+func (l Loader) IsVersion2() bool {
+	return types.ExtractBits(uint64(l.Info), 10, 1) != 0
+}
 
 func (l Loader) String() string {
 	var out []string
@@ -128,6 +134,9 @@ func (l Loader) String() string {
 	if l.IsPremapped() {
 		out = append(out, "premapped")
 	}
+	if l.IsVersion2() {
+		out = append(out, "v2")
+	}
 	return fmt.Sprintf("%s, ref: %s", strings.Join(out, "|"), l.Ref)
 }
 
@@ -303,6 +312,11 @@ type CodeSignatureInFile struct {
 	Size       uint32
 }
 
+type ExportsTrieLoaderInFile struct {
+	Offset uint64
+	Size   uint32
+}
+
 func deserializeAbsoluteValue(value uint64) uint64 {
 	// sign extend
 	if (value & 0x4000000000000000) != 0 {
@@ -317,7 +331,6 @@ type dependent struct {
 }
 
 type prebuiltLoaderHeader struct {
-	Loader
 	PathOffset                     uint16
 	DependentLoaderRefsArrayOffset uint16 // offset to array of LoaderRef
 	DependentKindArrayOffset       uint16 // zero if all deps normal
@@ -343,9 +356,9 @@ type prebuiltLoaderHeader struct {
 	IndexOfTwin          uint16 // if in dyld cache and part of unzippered twin, then index of the other twin
 	_                    uint16
 
-	ExportsTrieLoaderOffset uint64
-	ExportsTrieLoaderSize   uint32
-	VmSize                  uint32
+	ExportsTrieLoader ExportsTrieLoaderInFile
+
+	VmSize uint32
 
 	CodeSignature CodeSignatureInFile
 
@@ -449,7 +462,8 @@ func (o ObjCBinaryInfo) String() string {
 }
 
 type PrebuiltLoader struct {
-	prebuiltLoaderHeader
+	Loader
+	Header                      prebuiltLoaderHeader
 	Path                        string
 	AltPath                     string
 	Twin                        string
@@ -493,6 +507,9 @@ func (pl PrebuiltLoader) GetInfo() string {
 	if pl.IsCatalystOverride() {
 		out = append(out, "catalyst_override")
 	}
+	if pl.RegionsCount() > 0 {
+		out = append(out, fmt.Sprintf("regions=%d", pl.RegionsCount()))
+	}
 	return strings.Join(out, "|")
 }
 func (pl PrebuiltLoader) GetFileOffset(vmoffset uint64) uint64 {
@@ -514,9 +531,9 @@ func (pl PrebuiltLoader) String(f *File) string {
 	if pl.Twin != "" {
 		out += fmt.Sprintf("Twin:    %s\n", pl.Twin)
 	}
-	out += fmt.Sprintf("VM Size:       %#x\n", pl.VmSize)
-	if pl.CodeSignature.Size > 0 {
-		out += fmt.Sprintf("CodeSignature: off=%#08x, sz=%#x\n", pl.CodeSignature.FileOffset, pl.CodeSignature.Size)
+	out += fmt.Sprintf("VM Size:       %#x\n", pl.Header.VmSize)
+	if pl.Header.CodeSignature.Size > 0 {
+		out += fmt.Sprintf("CodeSignature: off=%#08x, sz=%#x\n", pl.Header.CodeSignature.FileOffset, pl.Header.CodeSignature.Size)
 	}
 	if pl.FileValidation != nil {
 		if pl.FileValidation.CheckCDHash {
@@ -538,11 +555,11 @@ func (pl PrebuiltLoader) String(f *File) string {
 	if len(pl.GetInfo()) > 0 {
 		out += fmt.Sprintf("Info:          %s\n", pl.GetInfo())
 	}
-	if pl.ExportsTrieLoaderSize > 0 {
-		out += fmt.Sprintf("ExportsTrie:   off=%#08x, sz=%#x\n", pl.GetFileOffset(pl.ExportsTrieLoaderOffset), pl.ExportsTrieLoaderSize)
+	if pl.Header.ExportsTrieLoader.Size > 0 {
+		out += fmt.Sprintf("ExportsTrie:   off=%#08x, sz=%#x\n", pl.GetFileOffset(pl.Header.ExportsTrieLoader.Offset), pl.Header.ExportsTrieLoader.Size)
 	}
-	if pl.FixupsLoadCommandOffset > 0 {
-		out += fmt.Sprintf("FixupsLoadCmd: off=%#08x\n", pl.FixupsLoadCommandOffset)
+	if pl.Header.FixupsLoadCommandOffset > 0 {
+		out += fmt.Sprintf("FixupsLoadCmd: off=%#08x\n", pl.Header.FixupsLoadCommandOffset)
 	}
 	if len(pl.Regions) > 0 {
 		out += "\nRegions:\n"
